/////////////////////////////////////////////////////////////////////////////
// Name:       Misc.h
// Purpose:    Misc stuff
// Part of:    4Pane
// Author:     David Hart
// Copyright:  (c) 2014 David Hart
// Licence:    wxWindows licence
/////////////////////////////////////////////////////////////////////////////
#ifndef MISCH
#define MISCH

#include "wx/wx.h"
#include "wx/dirctrl.h"

extern wxString GetCwd();
extern bool SetWorkingDirectory(const wxString& dir);
extern wxColour GetSaneColour(wxWindow* win, bool bg, wxSystemColour type);
extern void UnexecuteImages(const wxString& filepath) ;

// A class that displays a plain message, rather like wxMessageBox, but without buttons.  It autocloses after a user-defined no of seconds
class BriefMessageBox;

class MyBriefMessageTimer : public wxTimer  // Used by BriefMessageBox.  Notify() kills the BriefMessageBox when the timer 'fires'
{
public:
void Setup(wxDialog* ptrtobox ){ p_box = ptrtobox; }
protected:
void Notify(){ p_box->EndModal(wxID_CANCEL); }

wxDialog* p_box;
};

class BriefMessageBox : public wxDialog  // Displays a message dialog for a defined time only
{
public:
BriefMessageBox(wxString Message, double secondsdisplayed = -1, wxString Caption = wxEmptyString);
protected:
MyBriefMessageTimer BriefTimer;
};

enum  // for FileDirDlg
{
  FD_MULTIPLE = 0x0001,
  FD_SHOWHIDDEN = 0x0002,
  FD_RETURN_FILESONLY = 0x0004
};

class FileDirDlg  :  public wxDialog  // A dialog that can select both files and dirs
{
public:
FileDirDlg(){}
FileDirDlg(wxWindow* parent, int style=FD_MULTIPLE | FD_SHOWHIDDEN, wxString defaultpath=wxEmptyString, wxString label =_( "File and Dir Select Dialog" ));
wxString GetFilepath(){ return filepath; }              // Get a single filepath
size_t GetPaths(wxArrayString& filepaths);            // Get multiple filepaths. Returns number found

protected:
void OnSelChanged(wxTreeEvent& event);
void OnHome(wxCommandEvent& event);
void OnHidden(wxCommandEvent& event);
#if !defined(__WXGTK3__)
  void OnEraseBackground(wxEraseEvent& event);
#endif

wxString filepath;
wxGenericDirCtrl* GDC;
wxTextCtrl* Text;
int SHCUT_HOME;
bool filesonly;

DECLARE_DYNAMIC_CLASS(FileDirDlg)
};

class PasswordManager : public wxEvtHandler
{
public:
const wxString GetPassword(const wxString& cmd, wxWindow* caller = NULL); // Returns the password, first asking the user for it if need be
void ForgetPassword() { m_password.Clear(); } // Clear the stored pw, presumably because it was rejected by su

static PasswordManager& Get() { if (!ms_instance)	ms_instance = new PasswordManager; return *ms_instance; }
static void Release() { delete ms_instance; ms_instance = NULL; }

protected:
void SetPasswd(const wxString& password);   // Encrypts and stores the password
wxString GetPasswd() const;                 // Decrypts and returns the password

void OnPasswordTimer(wxTimerEvent& WXUNUSED(event)) { m_password.Clear(); }

wxTimer m_timer;
bool m_extendLife;                          // If true, extend the life of the password every time it's used
wxString m_password;
wxString m_key;

private:
PasswordManager() : wxEvtHandler()
    { m_timer.SetOwner(this);
      Connect(wxEVT_TIMER, wxTimerEventHandler(PasswordManager::OnPasswordTimer), NULL, this);
    }

static PasswordManager* ms_instance;
};


class ExecInPty  // Wrapper for ExecuteInPty()
{
#define ERROR_RETURN_CODE -1    // We're always executing wxEXEC_SYNC
#define CANCEL_RETURN_CODE -2   // If the user cancelled in the password dialog
public:
ExecInPty(wxTextCtrl* caller = NULL) : m_caller(caller) {}

long ExecuteInPty(const wxString& cmd);

wxTextCtrl* GetCallerTextCtrl() { return m_caller; } // Get calling textctrl. May return NULL
wxArrayString& GetOutputArray() { return m_outputArray; }
wxArrayString& GetErrorsArray() { return m_errorsArray; }

  // 'Process' interface
void AddInputString(const wxString& input) { m_inputArray.Add(input); } // User-input from caller
void WriteOutputToTextctrl();                        // The caller wants any pty output

protected:
void InformCallerOnTerminate();                      // Let the caller know we're finished
int CloseWithError(int fd, const wxString& msg, int errorcode = ERROR_RETURN_CODE); // Close on error or empty password

wxTextCtrl* m_caller;
wxArrayString m_inputArray;
wxArrayString m_outputArray;
wxArrayString m_errorsArray;
};

extern long ExecuteInPty(const wxString& cmd, wxArrayString& output, wxArrayString& errors);
extern long ExecuteInPty(const wxString& cmd);

#endif    //MISCH

